from abc import ABCMeta, abstractmethod
import re
import time
from selenium import webdriver
import os


class IPlotter(object):
    """Abstract IPlotter"""

    __metaclass__ = ABCMeta
    iframe = '<iframe srcdoc="{source}" src="" width="{w}" height="{h}" sandbox="allow-scripts"></iframe>'
    invalid_name_pattern = re.compile(r'[^a-zA-Z0-9_\-\. ]+')

    def __init__(self):
        super(IPlotter, self).__init__()

    @classmethod
    def is_valid_name(cls, name):
        '''
        check whether plot div id or filenname are valid
        '''
        if (cls.invalid_name_pattern.search(name)):
            return False
        else:
            return True

    @abstractmethod
    def render(self):
        '''
        render the data in HTML template
        '''
        pass

    @abstractmethod
    def plot(self):
        '''
        output an iframe containing the plot in the notebook without saving
        '''
        pass

    @abstractmethod
    def save(self):
        '''
        save the rendered html to a file in the same directory as the notebook
        '''
        pass

    @abstractmethod
    def plot_and_save(self):
        '''
        save the rendered html to a file and return an IFrame to display the
        plot in the notebook
        '''
        pass



from jinja2 import Template
from IPython.display import IFrame, HTML
import os
import json
from .base_plotter import IPlotter


class ChartJSPlotter(IPlotter):
    """
    Class for creating charts.js charts in ipython  notebook
    """

    head = '''
        <!-- Load Charts.js -->
       <script src='https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.5.0/Chart.bundle.min.js'></script>
    '''

    template = '''
        <canvas id='{{div_id}}'></canvas>
        <script>
            var ctx = document.getElementById('{{div_id}}').getContext('2d');
            ctx.canvas.width  = {{w}} - (.1 * {{w}});
            ctx.canvas.height = {{h}} - (.15 * {{h}});
            var myNewChart = new Chart(ctx,{ type: '{{chart_type}}', data: {{data}}, options: {{options}} });
        </script>
    '''

    def __init__(self):
        super(ChartJSPlotter, self).__init__()

    def render(self,
               data,
               chart_type,
               options=None,
               div_id="chart",
               head="",
               w=800,
               h=420):
        '''
        render the data in HTML template
        '''
        if not self.is_valid_name(div_id):
            raise ValueError(
                "Name {} is invalid. Only letters, numbers, '_', and '-' are permitted ".format(
                    div_id))

        return Template(head + self.template).render(
            div_id=div_id.replace(" ", "_"),
            data=json.dumps(
                data, indent=4).replace("'", "\\'").replace('"', "'"),
            chart_type=chart_type,
            options=json.dumps(
                options, indent=4).replace("'", "\\'").replace('"', "'"),
            w=w,
            h=h)

    def plot_and_save(self,
                      data,
                      chart_type,
                      options=None,
                      w=800,
                      h=420,
                      filename='chart',
                      overwrite=True):
        '''
        save the rendered html to a file and return an IFrame to display the plot in the notebook
        '''
        self.save(data, chart_type, options, filename, w, h, overwrite)
        return IFrame(filename + '.html', w, h)

    def plot(self, data, chart_type, options=None, w=800, h=420):
        '''
        output an iframe containing the plot in the notebook without saving
        '''
        return HTML(
            self.iframe.format(
                source=self.render(
                    data=data,
                    chart_type=chart_type,
                    options=options,
                    head=self.head,
                    w=w,
                    h=h),
                w=w,
                h=h))

    def save(self,
             data,
             chart_type,
             options=None,
             filename='chart',
             w=800,
             h=420,
             overwrite=True):
        '''
        save the rendered html to a file in the same directory as the notebook
        '''
        html = self.render(
            data=data,
            chart_type=chart_type,
            options=options,
            div_id=filename,
            head=self.head,
            w=w,
            h=h)

        if overwrite:
            with open(filename.replace(" ", "_") + '.html', 'w') as f:
                f.write(html)
        else:
            if not os.path.exists(filename.replace(" ", "_") + '.html'):
                with open(filename.replace(" ", "_") + '.html', 'w') as f:
                    f.write(html)
            else:
                raise IOError('File Already Exists!')



from .base_plotter import IPlotter
from .export import VirtualBrowser
from .c3_plotter import C3Plotter
from .plotly_plotter import PlotlyPlotter
from .chartjs_plotter import ChartJSPlotter
from .chartist_plotter import ChartistPlotter
from .google_plotter import GCPlotter
__version__ = '0.4.2'



from setuptools import setup
from setuptools import find_packages

__version__ = '0.4.2'

if __name__ == '__main__':

    setup(
        name='iplotter',
        version=__version__,
        description='JavaScript Charting for iPython/Jupyter notebooks',
        setup_requires=['setuptools-markdown'],
        long_description_markdown_filename='README.md',
        url='https://github.com/niloch/iplotter',
        author='csulliva',
        author_email='csulliva@brandeis.edu',
        license='MIT',
        packages=find_packages(),
        install_requires=[
            'Jinja2>=2.0', 'ipython>=3.0', 'notebook>=3.0', 'selenium>=3.0'
        ],
        zip_safe=False,
        include_package_data=True,
        keywords=['ipython', 'plotly', 'c3.js', 'plot', 'chart', 'plotly.js',
                  'charts.js', 'chartist.js', 'jupyter'],
        classifiers=[
            'Development Status :: 3 - Alpha',
            'Intended Audience :: Science/Research',
            'License :: OSI Approved :: MIT License',
            'Topic :: Scientific/Engineering :: Visualization',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
        ])



import time
from selenium import webdriver
import os


class VirtualBrowser(object):
    """Helper class for converting html charts to png"""

    def __init__(self, driver=webdriver.Chrome):
        super(VirtualBrowser, self).__init__()
        self.driver = driver()

    def __enter__(self):
        return self

    def save_as_png(self, filename, width=300, height=250, render_time=1):
        '''
        open saved html file in an virtual browser and save a screen shot to PNG format
        '''
        self.driver.set_window_size(width, height)
        self.driver.get('file://{path}/{filename}'.format(
            path=os.getcwd(), filename=filename + ".html"))
        time.sleep(render_time)
        self.driver.save_screenshot(filename + ".png")

    def __exit__(self, type, value, traceback):
        self.driver.quit()
        return True

    def quit(self):
        '''
        shutdown virtual browser when finished
        '''
        self.driver.quit()
        return True